package com.pj.tb_wallet_record;

import java.util.List;

import com.pj.tb_wallet_record.vo.WalletRecordVo;
import com.pj.utils.so.SoMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_wallet_record -- 服务点交易记录
 * @author plj 
 */
@RestController
@RequestMapping("/TbWalletRecord/")
public class TbWalletRecordController {

	/** 底层 Service 对象 */
	@Autowired
	TbWalletRecordService tbWalletRecordService;

	/** 增 */  
	@RequestMapping("add")
	@SaCheckPermission(TbWalletRecord.PERMISSION_CODE_ADD)
	public AjaxJson add(TbWalletRecord t){
		tbWalletRecordService.add(t);
		t = tbWalletRecordService.getById(SP.publicMapper.getPrimarykey());
		return AjaxJson.getSuccessData(t);
	}

	/** 删 */  
	@RequestMapping("delete")
	@SaCheckPermission(TbWalletRecord.PERMISSION_CODE_DEL)
	public AjaxJson delete(Long id){
		 tbWalletRecordService.delete(id);
		return AjaxJson.getSuccess();
	}
	
	/** 删 - 根据id列表 */  
	@RequestMapping("deleteByIds")
	@SaCheckPermission(TbWalletRecord.PERMISSION_CODE_DEL)
	public AjaxJson deleteByIds(){
		List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class); 
		int line = SP.publicMapper.deleteByIds(TbWalletRecord.TABLE_NAME, ids);
		return AjaxJson.getByLine(line);
	}
	
	/** 改 */  
	@RequestMapping("update")
	@SaCheckPermission(TbWalletRecord.PERMISSION_CODE_EDIT)
	public AjaxJson update(TbWalletRecord t){
		tbWalletRecordService.update(t);
		return AjaxJson.getSuccess();
	}

	/** 查 - 根据id */  
	@RequestMapping("getById")
		@SaCheckPermission(TbWalletRecord.PERMISSION_CODE)
	public AjaxJson getById(Long id){
		TbWalletRecord t = tbWalletRecordService.getById(id);
		return AjaxJson.getSuccessData(t);
	}

	/** 查集合 - 根据条件（参数为空时代表忽略指定条件） */  
	@RequestMapping("getList")
		@SaCheckPermission(TbWalletRecord.PERMISSION_CODE)
	public AjaxJson getList() { 
		SoMap so = SoMap.getRequestSoMap();
		List<WalletRecordVo> list = tbWalletRecordService.getList(so.startPage());
		return AjaxJson.getPageData(so.getDataCount(), list);
	}
	
	
	
	/** 改 - 删除状态(0=禁用,1=启用) */  
	@RequestMapping("updateDeleteStatus")
	@SaCheckPermission(TbWalletRecord.PERMISSION_CODE_EDIT)
	public AjaxJson updateDeleteStatus(Long id, String value){
		int line = SP.publicMapper.updateColumnById(TbWalletRecord.TABLE_NAME, "delete_status", value, id);
		return AjaxJson.getByLine(line);
	}
	
	

	
	
	

}
